/********************************************************************
* FileName:		FeatureDemoNode2.c
* Dependencies: none   
* Processor:	PIC18, PIC24, PIC32, dsPIC30, dsPIC33
*               tested with 18F4620, dsPIC33FJ256GP710	
* Complier:     Microchip C18 v3.04 or higher
*				Microchip C30 v2.03 or higher	
*               Microchip C32 v1.02 or higher
* Company:		Microchip Technology, Inc.
*
* Copyright and Disclaimer Notice for MiWi Software:
*
* Copyright  2007-2010 Microchip Technology Inc.  All rights reserved.
*
* Microchip licenses to you the right to use, modify, copy and distribute 
* Software only when embedded on a Microchip microcontroller or digital 
* signal controller and used with a Microchip radio frequency transceiver, 
* which are integrated into your product or third party product (pursuant 
* to the terms in the accompanying license agreement).   
*
* You should refer to the license agreement accompanying this Software for 
* additional information regarding your rights and obligations.
*
* SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY 
* KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY 
* WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A 
* PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE 
* LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, 
* CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY 
* DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO 
* ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, 
* LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, 
* TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT 
* NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*
*********************************************************************
* File Description:
*
*  This is the sample application that demonstrate the rich features
*  of MiWi(TM) Development Environment (DE). This demo should be used 
*  with FeatureDemoNode1 together. In this demo, following MiWi(TM) 
*  DE features have been implemented:
*   - Network Freezer
*       This application demonstrate how to invoke Network Freezer
*       feature that restore the previous network configurations
*       after reset or power cycle.
*   - Active Scan 
*       This application will do an active scan to allocate all PANs
*       running in the neighborhood and choose the PAN that share the 
*       same PAN identifier to establish connection.
*   - Sleep Feature 
*       This application is able to put the radio and MCU into sleep 
*       mode to conserve power. After waking up from the sleep, this
*       application is also capable of sending out Data Request command
*       to retrieve possible data from its associated node. The sleeping
*       period for this application is roughly 8 seconds, depends on 
*       the MCU family and operating frequency setting.
*   - Enhanced Data Request
*       This application is able to demonstrate the Enhanced Data 
*       Request feature, which combines Data Request command and the 
*       data message sending out from the sleeping device together. The
*       benefit is to reduce the message exchanges from 6 to 4, with
*       the total active time saving up to 30%. To use the Enhanced
*       Data Request feature, the outgoing message needs to be prepared
*       before the transceiver wakes up.
*   - Time Synchronization
*       When large amount of sleeping devices are connected to a single
*       Full Function Device (FFD), Time Synchronization feature helps 
*       to synchronize the check-in timing for each sleeping device, 
*       thus avoid packet collision, retransmission and undelivered 
*       messages.
*   - Frequency Agility 
*       As a frequency agility follower, this application is able to 
*       change operating channel when receiving a Channel Hopping 
*       command. This application is also capable of resynchronize
*       the connection by scanning possible channels.
*
* Change History:
*  Rev   Date         Author    Description
*  0.1   03/01/2008   yfy       Initial revision
*  3.1   5/28/2010    yfy       MiWi DE 3.1
********************************************************************/

/************************ HEADERS **********************************/
#include "ConfigApp.h"
//#include "Console.h"
#include "Transceivers\Transceivers.h"
#include "WirelessProtocols\SymbolTime.h"
#include "Transceivers\Security.h"
#include "WirelessProtocols\MCHP_API.h"

/************************ VARIABLES ********************************/
#define LIGHT   0x01
#define SWITCH  0x02

/*******************************************************************/
// AdditionalNodeID variable array defines the additional 
// information to identify a device on a P2P connection. This array
// will be transmitted with the P2P_CONNECTION_REQUEST command to 
// initiate the connection between the two devices. Along with the 
// long address of this device, this variable array will be stored 
// in the P2P Connection Entry structure of the partner device. The 
// size of this array is ADDITIONAL_CONNECTION_PAYLOAD, defined in 
// P2PDefs.h.
// In this demo, this variable array is set to be empty.
/******************************************************************/
#if ADDITIONAL_NODE_ID_SIZE > 0
    BYTE AdditionalNodeID[ADDITIONAL_NODE_ID_SIZE] = {SWITCH};
#endif

/*******************************************************************/
// The variable myChannel defines the channel that the P2P connection
// is operate on. This variable will be only effective if energy scan
// (ENABLE_ED_SCAN) is not turned on. Once the energy scan is turned
// on, the operating channel will be one of the channels available 
// with least amount of energy (or noise).
/*******************************************************************/
BYTE myChannel = 10; 



extern BYTE myLongAddress[];
/***************************************************************************
* Function:         void main(void)
*
* PreCondition:     none
*
* Input:		    none
*
* Output:		    none
*
* Side Effects:	    none
*
* Overview:		    This is the main function that runs the demo.  
*                   The device will first search for a P2P connection
*                   Once the connection is established, pressing 
*                   button one (RB5 on PICDEM Z or RD6 on Explorer 16) 
*                   or button two (RB4 on PICDEM Z or RD7 on Explorer16) 
*                   will send out broadcast and unicast packets.
*
* Note:			    
****************************************************************************/
#if defined(__18CXX)
void main(void)
#else
int main(void)
#endif
{   
    BYTE i, j;
    BYTE OperatingChannel = 0xFF;
    BYTE TxSynCount = 0;
    BYTE TxSynCount2 = 0;
    //TICK t1, t2;
    BYTE TxPersistFailures = 0;
    BOOL ReadyToSleep = FALSE;
    BYTE TxNum = 0;
    BYTE RxNum = 0;
    BYTE PressedButton = 0;
    
    /*******************************************************************/
    // Initialize the system
    /*******************************************************************/
    BoardInit();         
    ConsoleInit();  
    LED_1 = 0;
    LED_2 = 0;
    
    
    /*******************************************************************/
    // Following block display demo information on LCD of demo board.
    /*******************************************************************/    
    #if defined(MRF49XA)
        #if defined(PROTOCOL_P2P)
            LCDDisplay((char *)"Feature P2P Demo MRF49XA Node 2", 0, TRUE); 
        #endif
        #if defined(PROTOCOL_MIWI)
            LCDDisplay((char *)"Feature MiWi Dem MRF49XA Node 2", 0, TRUE); 
        #endif
    #elif defined(MRF24J40)
        #if defined(PROTOCOL_P2P)
            LCDDisplay((char *)"Feature P2P DemoMRF24J40 Node 2", 0, TRUE);
        #endif
        #if defined(PROTOCOL_MIWI)
            LCDDisplay((char *)"Feature MiWi DemMRF24J40 Node 2", 0, TRUE);
        #endif
    #elif defined(MRF89XA)
        #if defined(PROTOCOL_P2P)
            LCDDisplay((char *)"Feature P2P Demo MRF89XA Node 2", 0, TRUE); 
        #endif
        #if defined(PROTOCOL_MIWI)
            LCDDisplay((char *)"Feature MiWi Dem MRF89XA Node 2", 0, TRUE); 
        #endif
    #endif

    /*******************************************************************/
    // If button 1 on the demo board is pressed and hold during the
    // reset or power on process, Network Freezer feature is invoked
    // to restore all previous network configurations. If Network 
    // Freezer feature is invoked, there is no requirement to re-
    // establish new connection
    /*******************************************************************/
    if( PUSH_BUTTON_1 == 0 )
    {  
        while(PUSH_BUTTON_1 == 0 );
        
        /*********************************************************************/
        // Function MiApp_ProtocolInit intialize the protocol stack.
        // The return value is a boolean to indicate the status of the 
        //      operation.
        // The only parameter indicates if Network Freezer should be invoked.
        //      When Network Freezer feature is invoked, all previous network
        //      configurations will be restored to the states before the 
        //      reset or power cycle
        //
        // In this example, we assume that the user wants to apply Network
        //      Freezer feature and restore the network configuration if
        //      button 1 is pressed and hold when powering up.
        /*********************************************************************/
        MiApp_ProtocolInit(TRUE);

        LCDDisplay((char *)" Network Freezer    ENABLED", 0, TRUE); 

    }
    else
    {
        /*********************************************************************/
        // Function MiApp_ProtocolInit intialize the protocol stack.
        // The return value is a boolean to indicate the status of the 
        //      operation.
        // The only parameter indicates if Network Freezer should be invoked.
        //      When Network Freezer feature is invoked, all previous network
        //      configurations will be restored to the states before the 
        //      reset or power cycle
        //
        // In this example, we assume that the user wants to start from
        // scratch and ignore previous network configuration if button 1
        // is released when powering up.
        /*********************************************************************/
        MiApp_ProtocolInit(FALSE);   
                   
        LCDDisplay((char *)"Active Scanning", 0, FALSE);
            
		

		
        while(OperatingChannel == 0xFF)
        {

	/*
	* Masters 2010 MiWiLab:  ActiveScan
	*/

                
        }        
                


      
        /*******************************************************************/
        // Display current opertion on LCD of demo board, if applicable
        /*******************************************************************/
        LCDDisplay((char *)" Connected Peer  on Channel %d", currentChannel, TRUE);
        
    }
    
    // Turn on LED 1 to indicate P2P connection established
    LED_1 = 1;

    /*******************************************************************/
    // Following block display demo instructions on LCD based on the
    // demo board used. 
    /*******************************************************************/
    #if defined(EXPLORER16)
        LCDDisplay((char *)"RD6: Broadcast  RD7: Unicast", 0, FALSE); 
    #elif defined(PIC18_EXPLORER)
        LCDDisplay((char *)"RB0: Broadcast", 0, FALSE); 
    #elif defined(EIGHT_BIT_WIRELESS_BOARD)
        LCDDisplay((char *)"RB0: Broadcast  RB2: Unicast", 0, FALSE);
    #endif

    while(1)
    {   
        /*******************************************************************/
        // Function MiApp_MessageAvailable will return a boolean to indicate 
        // if a message for application layer has been received by the 
        // transceiver. If a message has been received, all information will 
        // be stored in rxMessage, structure of RECEIVED_MESSAGE.
        /*******************************************************************/
        if( MiApp_MessageAvailable() )
        {
            /*******************************************************************/
            // If a packet has been received, following code prints out some of
            // the information available in rxMessage.
            /*******************************************************************/
            
            // Toggle LED2 to indicate receiving a packet.
            LED_2 ^= 1;

            /*******************************************************************/
            // Function MiApp_DiscardMessage is used to release the current 
            // received message. After calling this function, the stack can 
            // start to process the next received message.
            /*******************************************************************/ 
            MiApp_DiscardMessage();
            
            /*******************************************************************/
            // Following block update the total received and transmitted 
            // messages on the LCD of the demo board. 
            /*******************************************************************/
            LCDTRXCount(TxNum, ++RxNum);
        }
        else 
        {
            #ifdef ENABLE_FREQUENCY_AGILITY
                /***********************************************************************/
                // TxPersistFailures is the local variable to track the transmission 
                // failure because no acknowledgement frame is received. Typically,
                // this is the indication of either very strong noise, or the PAN
                // has hopped to another channel. 
                /***********************************************************************/
                if( TxPersistFailures > 3 )
                {
                    /***********************************************************************/
                    // Display the resynchronization message on LCD of demo board, if 
                    // applicable 
                    /***********************************************************************/
                    LCDDisplay((char *)"Resynchronizing the Connection", 0, FALSE); 
                    
                    /*******************************************************************/
                    // Function MiApp_TransceiverPowerState is used to set the power state
                    // of RF transceiver. There are three possible states:
                    //   - POWER_STATE_SLEEP        Put transceiver into sleep
                    //   - POWER_STATE_WAKEUP       Wake up the transceiver only
                    //   - POWER_STATE_WAKEUP_DR    Wake up and send Data Request command
                    /*******************************************************************/
                    MiApp_TransceiverPowerState(POWER_STATE_WAKEUP);
                    
                    /***********************************************************************/
                    // Function MiApp_ResyncConnection is used to synchronized connection
                    // if one side of communication jumped to another channel, when 
                    // frequency agility is performed. Usually, this is done by the 
                    // sleeping device, since the sleeping device cannot hear the broadcast 
                    // of channel hopping command.
                    //
                    // The first parameter is the index of connection table for the peer 
                    // node, which we would like to resynchronize to.
                    // The second parameter is the bit map of channels to be scanned
                    /***********************************************************************/
                    MiApp_ResyncConnection(0, 0xFFFFFFFF);    
                    TxPersistFailures = 0;
                    ReadyToSleep = FALSE;
                    /***********************************************************************/
                    // Display the resynchronization result on the LCD of demo board, if 
                    // applicable
                    /***********************************************************************/
                    LCDDisplay((char *)" Resynchronized  to Channel %d", currentChannel, FALSE);
                }
                else
                {
                    #ifdef ENABLE_SLEEP
                        ReadyToSleep = TRUE;
                    #endif
                }    
            
            #endif
            
            #ifdef ENABLE_SLEEP
                /*******************************************************************/
                // If Data Request command and data transmision has been handled,
                // as the RFD device, it is time to consider put both the radio and 
                // MCU into sleep mode to conserve power.
                /*******************************************************************/
                if( ReadyToSleep )
                {   
                    #if defined(ENABLE_TIME_SYNC)
                        DWORD_VAL    delayTick;
                    #endif
                    ReadyToSleep = FALSE;
                    
                    /*******************************************************************/
                    // Function MiApp_TransceiverPowerState is used to set the power 
                    // state of RF transceiver. There are three possible states:
                    //   - POWER_STATE_SLEEP        Put transceiver into sleep
                    //   - POWER_STATE_WAKEUP       Wake up the transceiver only
                    //   - POWER_STATE_WAKEUP_DR    Wake up and send Data Request 
                    //                              command
                    /*******************************************************************/
                    MiApp_TransceiverPowerState(POWER_STATE_SLEEP);
                                        
                    /*******************************************************************/
                    // Prepare the condition to wake up the MCU. The MCU can either be
                    // waken up by the timeout of watch dog timer (Time Synchronization 
                    // off), timeout of counter with external 32KHz crystal (Time 
                    // Synchronization on), or by the pin change notification interrupt 
                    // by pushing the button.
                    /*******************************************************************/
                    #if defined(ENABLE_TIME_SYNC)
                        /******************************************************************/
                        // When Time Synchronization feature is enabled, a counter with 
                        // external 32KHz crystal will be used to keep the precise timing
                        // when the MCU is in sleep. we need to update the counter with 
                        // proper value. The timeout of the counter will wake up the MCU.
                        /******************************************************************/
                        
                        /******************************************************************/
                        // Following block is used to update the CounterValue with more
                        // precise timing control. This block is useful if very precise 
                        // timing is required, such as time slot is very short (<< 1 second)
                        // or application spends considerable, unpredictable time performing
                        // other operations
                        /******************************************************************/
                        #if 1
                            delayTick.Val = 0;
                            
                            #if defined(__18CXX)
                                delayTick.v[0] = TMR3L;
                                delayTick.v[1] = TMR3H;
                            #elif defined(__dsPIC33F__) || defined(__PIC24F__) || defined(__PIC24FK__) ||defined(__PIC24H__)
                                delayTick.w[0] = TMR1;
                            #elif defined(__PIC32MX__)
                                delayTick.w[0] = TMR1;
                            #endif
                            delayTick.Val += CounterValue.Val;
                            if( delayTick.v[2] > 0 )
                            {
                                if( WakeupTimes.Val > 0 )
                                {
                                    WakeupTimes.Val--;
                                }
                                else
                                {
                                    delayTick.Val = CounterValue.Val;
                                }    
                            }
                            CounterValue.v[0] = delayTick.v[0];
                            CounterValue.v[1] = delayTick.v[1];   
                        #endif 
                        
                        /******************************************************************/
                        // The total sleeping time of the device is controlled by two 16bit
                        // parameters: WakeupTimes and CouterValue. Both values are referred
                        // from the protocol stack.
                        // The counter with external 32KHz crystal can provide wakeup time 
                        // up to around 16 seconds. 
                        // If the intended sleep period is less than 16 second, only 
                        // CounterValue will be loaded to the counter, the device
                        // will wake up by the counter timeout. 
                        // If the intended sleep period is longer than 16 seconds, 
                        // WakeupTimes will store the total number of 16-second wakeup and 
                        // the rest remaining time will be loaded into CounterValue. When
                        // the counter with external 32KHz crystal timeout and wake up the
                        // MCU, the MCU will reduce WakeupTimes by one and keep the radio
                        // in sleep. When the WakeupTimes are reduced to be zero, 
                        // CounterValue will be loaded to the counter. The whole device
                        // will wake up once the counter timeout this time.
                        /******************************************************************/
                        while(1)
                        {
                            #if defined(__18CXX)
                                if( WakeupTimes.Val > 0 )
                                {
                                    LED_2 = 0;
                                    TMR3H = 0;
                                    TMR3L = 0;
                                }    
                                else
                                {
                                    LED_2 = 1;
                                    
                                    TMR3H = CounterValue.v[1];
                                    TMR3L = CounterValue.v[0];
                                }    
                                PIR2bits.TMR3IF = 0;
                                while( !ConsoleIsPutReady() );
                                #if defined(PIC18_EXPLORER)
                                    INTCONbits.INT0IF = 0;
                                    INTCON2bits.INTEDG0 = 0;
                                    INTCONbits.INT0IE = 1;
                                #elif defined(EIGHT_BIT_WIRELESS_BOARD)
                                    INTCONbits.INT0IF = 0;
                                    INTCON2bits.INTEDG0 = 0;
                                    INTCONbits.INT0IE = 1;
                                    INTCON3bits.INT2IF = 0;
                                    INTCON2bits.INTEDG2 = 0;
                                    INTCON3bits.INT2IE = 1;
                                #elif defined(PICDEMZ)
                                    INTCONbits.RBIF = 0;        // clear pin change notification interrupt
                                    INTCONbits.RBIE = 1;        // enable pin change notification interrupt
                                #endif
                            #elif defined(__dsPIC33F__) || defined(__PIC24F__) || defined(__PIC24FK__) || defined(__PIC24H__)
                                
                                if( WakeupTimes.Val > 0 )
                                {
                                    LED_2 = 0;
                                    TMR1 = 0;
                                }    
                                else
                                {
                                    LED_2 = 1;
                                    
                                    TMR1 = CounterValue.Val;
                                }  
                                IFS0bits.T1IF = 0;

                                IFS1bits.CNIF = 0;
                                IEC1bits.CNIE = 1;          // enable pin change notification interrupt
                            #elif defined(__PIC32MX__)
                                if( WakeupTimes.Val > 0 )
                                {
                                    LED_2 = 0;
                                    TMR1 = 0;
                                }    
                                else
                                {
                                    LED_2 = 1;
                                    while(T1CONbits.TWIP) ;
                                    TMR1 = CounterValue.Val;
                                }  
                                IFS0bits.T1IF = 0;
                              
                                Enable_PB_1_2_Interrupts(); // enable pin change notification interrupt
                            #endif
                            
                            // make sure UART has nothing to output
                            while(ConsoleIsPutReady() == 0);
                    
                            // Put MCU into sleep mode
                            Sleep();
                            
                            #if defined(__18CXX)
                                #if defined(PIC18_EXPLORER)
                                    INTCONbits.INT0IE = 0;
                                #elif defined(EIGHT_BIT_WIRELESS_BOARD)
                                    INTCONbits.INT0IE = 0;
                                    INTCON3bits.INT2IE = 0;    
                                #elif defined(PICDEMZ)
                                    INTCONbits.RBIE = 0;        // disable pin change notification interrupt
                                #endif
                            #elif defined(__dsPIC33F__) || defined(__PIC24F__) || defined(__PIC24FK__) || defined(__PIC24H__)
                                //IEC1bits.CNIE = 0;          // disable pin change notification interrupt
                            #elif defined(__PIC32MX__)
                                RCONCLR = RCON_SLEEP_MASK | RCON_IDLE_MASK;   // clear the Sleep and Idle bits
                            #endif
                            
                            if( WakeupTimes.Val == 0 )
                            {
                                break;
                            }    
                            WakeupTimes.Val--;
                        } 
                    
                    #else
                    
                        /******************************************************************/
                        // When Time Synchronization is not enabled, watch dog timer is 
                        // used to wake up the device periodically.
                        /******************************************************************/
                        #if defined(__18CXX)
                            ClrWdt();
                            WDTCONbits.SWDTEN = 1;      // enable watch dog timer
                            #if defined(PIC18_EXPLORER)
                                INTCONbits.INT0IF = 0;
                                INTCON2bits.INTEDG0 = 0;
                                INTCONbits.INT0IE = 1;
                            #elif defined(EIGHT_BIT_WIRELESS_BOARD)
                                INTCONbits.INT0IF = 0;
                                INTCON2bits.INTEDG0 = 0;
                                INTCONbits.INT0IE = 1;
                                INTCON3bits.INT2IF = 0;
                                INTCON2bits.INTEDG2 = 0;
                                INTCON3bits.INT2IE = 1;
                            #elif defined(PICDEMZ)
                                INTCONbits.RBIF = 0;        // clear pin change notification interrupt
                                INTCONbits.RBIE = 1;        // enable pin change notification interrupt
                            #endif
                        #elif defined(__dsPIC33F__) || defined(__PIC24F__) || defined(__PIC24FK__) || defined(__PIC24H__)
                            ClrWdt();
                            RCONbits.SWDTEN = 1;        // enable watch dog timer
                            IFS1bits.CNIF = 0;
                            IEC1bits.CNIE = 1;          // enable pin change notification interrupt
                        #elif defined(__PIC32MX__)
                            Enable_PB_1_2_Interrupts(); // enable pin change notification interrupt
                            EnableWDT();                // enable watch dog timer
                            ClearWDT();                 // clear watch dog timer
                        #endif
                        
                        // make sure UART has nothing to output
//                        while(ConsoleIsPutReady() == 0);
                        
                        // Put MCU into sleep mode
                        Sleep();
                        
                        #if defined(__18CXX)
                            #if defined(PIC18_EXPLORER)
                                INTCONbits.INT0IE = 0;
                            #elif defined(EIGHT_BIT_WIRELESS_BOARD)
                                INTCONbits.INT0IE = 0;
                                INTCON3bits.INT2IE = 0;    
                            #elif defined(PICDEMZ)
                                INTCONbits.RBIE = 0;        // disable pin change notification interrupt
                            #endif
                            WDTCONbits.SWDTEN = 0;      // disable watch dog timer
                        #elif defined(__dsPIC33F__) || defined(__PIC24F__) || defined(__PIC24FK__) || defined(__PIC24H__)
                            RCONbits.SWDTEN = 0;        // disable watch dog timer                           
                        #elif defined(__PIC32MX__)
                            RCONCLR = RCON_SLEEP_MASK | RCON_IDLE_MASK;   // clear the Sleep and Idle bits
                            DisableWDT();
                        #endif
                    #endif

                    
                    /*******************************************************************/
                    // Function ButtonPressed will return if any of the two buttons
                    // (RB5 and RB4 on PICDEM Z or RD6 and RD7 on Explorer 16) has been
                    // pushed.
                    /*******************************************************************/
                    PressedButton = ButtonPressed();   

                    #if defined(ENABLE_ENHANCED_DATA_REQUEST) 
                        /*******************************************************************/
                        // The handling of Enhanced Data Request feature depends on when
                        // to send out message. If Enhanced Data Request feature is enabled,
                        // The message should be sent out before transceiver wakes up and
                        // sends out the Data Reqeust. Otherwise, the message should be 
                        // sent after transceiver wakes up and sends out the Data Reqeust 
                        // command.
                        // When Enhanced Data Request feature is enabled, only unicast 
                        // message can be sent with Data Request. Broadcast, however, has 
                        // a different destination address, thus must be sent after the 
                        // normal Data Request procedure in the normal way. 
                        /*******************************************************************/         
                        switch( PressedButton )
                        {
                            case 1: 
                                /*******************************************************************/                
                                // Button 1 pressed
                                // First use MiApp_FlushTx to reset the Transmit buffer. Then fill 
                                // the TX buffer by calling function MiApp_WriteData
                                /*******************************************************************/
                                MiApp_FlushTx();
                                if( (TxSynCount % 6 ) == 0 )
                                {
                                    MiApp_WriteData(0x0D);
                                    MiApp_WriteData(0x0A);
                                }
                                else
                                {                     
                                    for(i = 0; i < 21; i++)
                                    {
                                        MiApp_WriteData(MiWi[(TxSynCount%6)-1][i]);
                                    }
                                }
                                TxSynCount++;
                                
                                /*******************************************************************/
                                // Function MiApp_UnicastConnection is one of the functions to 
                                // unicast a message.
                                //    The first parameter is the index of connection table for 
                                //       the peer device. In this demo, since there are only two
                                //       devices involved, the peer device must be stored in the 
                                //       first P2P Connection Entry in the connection table.
                                //    The second parameter is the boolean to indicate if we need 
                                //       to secure the frame. If encryption is applied, the 
                                //       security level and security key are defined in the 
                                //       configuration file for the transceiver
                                //
                                // Another way to unicast a message is by calling function
                                // MiApp_UnicastAddress. Instead of supplying the index of the 
                                // connection table of the peer device, this function requires 
                                // the input parameter of destination address directly.
                                /*******************************************************************/
                                if( MiApp_UnicastConnection(0, FALSE) == FALSE )
                                {
                                    Printf("\r\nUnicast Failed\r\n");
                                    /*******************************************************************/
                                    // Following block display error message on LCD of demo board.
                                    /*******************************************************************/
                                    LCDDisplay((char *)" Unicast Failed", 0, TRUE);
                                }
                                else
                                {
                                    TxNum++;
                                }
                                
                                /*******************************************************************/
                                // Following block update the total received and transmitted messages
                                // on the LCD of the demo board. 
                                /*******************************************************************/
                                LCDTRXCount(TxNum, RxNum);   

                                break;
              
                            case 2:
                                /*******************************************************************/                
                                // Button 2 pressed.
                                // First use MiApp_FlushTx to reset the Transmit buffer. Then fill 
                                // the TX buffer by calling function MiApp_WriteData
                                /*******************************************************************/
                                MiApp_FlushTx();
                                if( (TxSynCount2 % 6 ) == 0 )
                                {
                                    MiApp_WriteData(0x0D);
                                    MiApp_WriteData(0x0A);
                                }
                                else
                                {                     
                                    for(i = 0; i < 11; i++)
                                    {
                                        MiApp_WriteData(DE[(TxSynCount2%6)-1][i]);
                                    }
                                }
                                TxSynCount2++;
                                
                                /*******************************************************************/
                                // Function MiApp_UnicastConnection is one of the functions to 
                                // unicast a message.
                                //    The first parameter is the index of connection table for 
                                //       the peer device. In this demo, since there are only two
                                //       devices involved, the peer device must be stored in the 
                                //       first P2P Connection Entry in the connection table.
                                //    The second parameter is the boolean to indicate if we need 
                                //       to secure the frame. If encryption is applied, the 
                                //       security level and security key are defined in the 
                                //       configuration file for the transceiver
                                //
                                // Another way to unicast a message is by calling function
                                // MiApp_UnicastAddress. Instead of supplying the index of the 
                                // connection table of the peer device, this function requires 
                                // the input parameter of destination address directly.
                                /*******************************************************************/
                                if( MiApp_UnicastConnection(0, TRUE) == FALSE )
                                {
                                    Printf("\r\nUnicast Failed\r\n");
                                    /*******************************************************************/
                                    // Following block display error message on LCD of demo board.
                                    /*******************************************************************/
                                    LCDDisplay((char *)" Unicast Failed", 0, TRUE);
                                }
                                else
                                {
                                    TxNum++;
                                }
                                
                                /*******************************************************************/
                                // Following block update the total received and transmitted messages
                                // on the LCD of the demo board. 
                                /*******************************************************************/
                                LCDTRXCount(TxNum, RxNum);
                                break;
                                
                            default:
                                MiApp_FlushTx();
                                break;
                        }
   
                    #endif
                    
                    /*******************************************************************/
                    // Function MiApp_TransceiverPowerState is used to set the power 
                    // state of RF transceiver. There are three possible states:
                    //   - POWER_STATE_SLEEP        Put transceiver into sleep
                    //   - POWER_STATE_WAKEUP       Wake up the transceiver only
                    //   - POWER_STATE_WAKEUP_DR    Wake up and send Data Request 
                    //                              command
                    /*******************************************************************/
                    if( MiApp_TransceiverPowerState(POWER_STATE_WAKEUP_DR) > SUCCESS )
                    {
                        TxPersistFailures++;
                        LCDDisplay((char *)"Connection Lost     %d Times", TxPersistFailures, FALSE);
                    }
                    else
                    {
                        if( TxPersistFailures > 0 )
                        {
                            LCDTRXCount(TxNum, RxNum);
                        }
                        TxPersistFailures = 0;
                        
                    }
                    
                    #if !defined(ENABLE_ENHANCED_DATA_REQUEST)
                        /*******************************************************************/
                        // The handling of Enhanced Data Request feature depends on when
                        // to send out message. If Enhanced Data Request feature is enabled,
                        // The message should be sent out before transceiver wakes up and
                        // sends out the Data Reqeust. Otherwise, the message should be 
                        // sent after transceiver wakes up and sends out the Data Reqeust 
                        // command.
                        /*******************************************************************/
                        switch( PressedButton )
                        {
                            case 1: 
                                /*******************************************************************/                
                                // Button 1 pressed.
                                // First use MiApp_FlushTx to reset the Transmit buffer. Then fill 
                                // the TX buffer by calling function MiApp_WriteData
                                /*******************************************************************/



				/*
				* Masters 2010 MiWiLab:  Broadcast
				*/


				/*
				* 
				*/
				//   

                                /*******************************************************************/
                                // Following block update the total received and transmitted 
                                // messages on the LCD of the demo board. 
                                /*******************************************************************/
                                LCDTRXCount(++TxNum, RxNum);   
                                break;
              
                            case 2:
                                /*******************************************************************/                
                                // Button 2 pressed.
                                // First use MiApp_FlushTx to reset the Transmit buffer. Then fill 
                                // the TX buffer by calling function MiApp_WriteData
                                /*******************************************************************/



				/*
				* Masters 2010 MiWiLab:  Unicast
				*/
                    

				/*
				* 
				*/
                                
                                /*******************************************************************/
                                // Following block update the total received and transmitted 
                                // messages on the LCD of the demo board. 
                                /*******************************************************************/
                                LCDTRXCount(TxNum, RxNum);
                                break;
                                
                            default:
                                break;
                        }

                    #endif

                }
            #endif   
             
        }
    }
}


